@@ -16,7 +16,7 @@ from urllib3.connection import HTTPConnection
 from oauthlib.oauth2 import BackendApplicationClient, TokenExpiredError
 from requests_oauthlib import OAuth2Session
 
-from . import utils
+from . import utils, entity_json
 from .dataset import Dataset
 from .exceptions import PipeAlreadyExists, SystemAlreadyExists, ConfigUploadFailed, \
     TimeoutWhileWaitingForBackupToFinishException, ServiceUnavailable, InternalServerError, BadRequestException
@@ -218,6 +218,7 @@ class Connection:
         return kwargs
 
     def do_put_request(self, url, data=None, allowable_response_status_codes=(200,), retries=None, retry_delay=None,
+                       files=None,
                        **kwargs):
         assert url is not None
         session_put_kwargs = self._get_requests_kwargs()
@@ -232,12 +233,12 @@ class Connection:
         while True:
             try:
                 try:
-                    response = self.session.put(url, data=data, **session_put_kwargs)
+                    response = self.session.put(url, data=data, files=files, **session_put_kwargs)
                 except TokenExpiredError:
                     # Manually clear the token and retry once. The token is supposed to be automatically
                     # refreshed, but if the local clock is off that can mess things up.
                     self._oauth2_token = None
-                    self._rewind_request_data_if_needed(data)
+                    self._rewind_request_data_if_needed(data, files)
                     response = self.session.put(url, data=data, **session_put_kwargs)
 
                 if allowable_response_status_codes:
@@ -249,7 +250,7 @@ class Connection:
                 if retries <= 0:
                     raise e
 
-                self._rewind_request_data_if_needed(data)
+                self._rewind_request_data_if_needed(data, files)
 
                 retries -= 1
                 self.interruptable_sleep(retry_delay)
@@ -355,7 +356,9 @@ class Connection:
                 self.interruptable_sleep(retry_delay)
 
     def do_post_request(self, url, allowable_response_status_codes=(200,), retries=None, retry_delay=None,
-                        data=None, **kwargs):
+                        data=None,
+                        files=None,
+                        **kwargs):
         assert url is not None
         session_post_kwargs = self._get_requests_kwargs()
         session_post_kwargs.update(kwargs)
@@ -369,12 +372,12 @@ class Connection:
         while True:
             try:
                 try:
-                    response = self.session.post(url, data=data, **session_post_kwargs)
+                    response = self.session.post(url, data=data, files=files, **session_post_kwargs)
                 except TokenExpiredError:
                     # Manually clear the token and retry once. The token is supposed to be automatically
                     # refreshed, but if the local clock is off that can mess things up.
                     self._oauth2_token = None
-                    self._rewind_request_data_if_needed(data)
+                    self._rewind_request_data_if_needed(data, files)
                     response = self.session.post(url, data=data, **session_post_kwargs)
 
                 if allowable_response_status_codes:
@@ -386,15 +389,25 @@ class Connection:
                 if retries <= 0:
                     raise e
 
-                self._rewind_request_data_if_needed(data)
+                self._rewind_request_data_if_needed(data, files)
 
                 retries -= 1
                 self.interruptable_sleep(retry_delay)
 
-    def _rewind_request_data_if_needed(self, data):
+    def _rewind_request_data_if_needed(self, data, files):
         if isinstance(data, io.BytesIO):
             data.seek(0)
 
+        if files:
+            for name, fileinfo in files.items():
+                if type(fileinfo) is tuple:
+                    # This is a filetuple on the form ('filename', fileobj)
+                    fileobj = fileinfo[1]
+                else:
+                    fileobj = fileinfo
+                if isinstance(fileobj, io.BytesIO):
+                    fileobj.seek(0)
+
     def do_delete_request(self, url, allowable_response_status_codes=(200,), retries=None, retry_delay=None, **kwargs):
         assert url is not None
         session_delete_kwargs = self._get_requests_kwargs()
@@ -1077,7 +1090,7 @@ class Connection:
 
         return parsed_response
 
-    def get_access_control_list(self, acl_path, **kwargs):
+    def get_access_control_list(self, acl_path, do_transit_decoding=True, **kwargs):
         """Retrieves the specified acl. The acl_path is the relative part of the url to the acl.
 
         Examples:
@@ -1088,7 +1101,7 @@ class Connection:
         acl_url = self.sesamapi_base_url + "permissions/" + acl_path
         response = self.do_get_request(acl_url, allowable_response_status_codes=[200], **kwargs)
         utils.validate_equal_case_insensitive(response.headers["Content-Type"], "application/json")
-        acl = utils.parse_json_response(response)
+        acl = entity_json.parse_entity(response.content.decode("utf-8"), do_transit_decoding=do_transit_decoding)
         return acl
 
     def set_access_control_list(self, acl_path, acl, **kwargs):
