@@ -232,7 +232,13 @@ class Pipe(EntityBase):
             if "state" not in self.runtime:
                 raise AssertionError(f"No 'state' property found in self.runtime! self.runtime: {runtime}")
             if self.runtime["state"] != "Deploying":
-                return
+                try:
+                    self.get_pump()
+                    return
+                except:
+                    logger.warning(f"The pipe with id '{self.id}' has been deployed, but an error was raised while "
+                                   f"accessing the pump. Will retry.")
+                    pass
             elapsetime = time.monotonic() - starttime
             if elapsetime > timeout:
                 raise AssertionError(
