@@ -42,7 +42,7 @@ def validate_response_is_ok(response, allowable_response_status_codes=frozenset(
             response=response
         )
 
-    if is_called_by_nose():
+    if is_called_by_unittest():
         # The sesamclient is being used by a ci-test, so do some additional checks.
         # check for headers with duplicate values
 
@@ -63,7 +63,7 @@ def validate_response_is_ok(response, allowable_response_status_codes=frozenset(
                         header_values_set.add(value)
 
 
-def parse_json_response(response):
+def parse_json_response(response, ):
     """Utility-function for parsing a http response as json and reporting any parsing-errors in a human-readable
     way. The response.json() method doesn't give very helpful error-messages.
 
@@ -103,6 +103,9 @@ def get_version():
         return version_file.read().strip()
 
 
-def is_called_by_nose():
+def is_called_by_unittest():
     stack = inspect.stack()
-    return any(x[0].f_globals['__name__'].startswith('nose.') for x in stack if x)
+    if any(x[0].f_globals['__name__'].startswith('nose.') for x in stack):
+        return True
+
+    return any(x[0].f_globals['__name__'].startswith('_pytest') for x in stack)
